@@ -3,8 +3,8 @@
 /**
  * @file classes/oai/omp/OAIDAO.inc.php
  *
- * Copyright (c) 2014-2020 Simon Fraser University
- * Copyright (c) 2003-2020 John Willinsky
+ * Copyright (c) 2014-2021 Simon Fraser University
+ * Copyright (c) 2003-2021 John Willinsky
  * Distributed under the GNU GPL v3. For full terms see the file docs/COPYING.
  *
  * @class OAIDAO
@@ -74,21 +74,21 @@ class OAIDAO extends PKPOAIDAO {
 
 	/**
 	 * Return hierarchy of OAI sets (presses plus press series).
-	 * @param $pressId int
+	 * @param $pressId int|null
 	 * @param $offset int
 	 * @param $total int
 	 * @return array OAISet
 	 */
-	function getSets($pressId = null, $offset, $limit, &$total) {
+	function getSets($pressId, $offset, $limit, &$total) {
 		if (isset($pressId)) {
-			$presses = array($this->getPress($pressId));
+			$presses = [$this->getPress($pressId)];
 		} else {
 			$pressFactory = $this->_pressDao->getAll();
 			$presses = $pressFactory->toArray();
 		}
 
 		// FIXME Set descriptions
-		$sets = array();
+		$sets = [];
 		foreach ($presses as $press) {
 			$title = $press->getLocalizedName();
 			$abbrev = $press->getPath();
@@ -110,7 +110,7 @@ class OAIDAO extends PKPOAIDAO {
 			}
 		}
 
-		HookRegistry::call('OAIDAO::getSets', array(&$this, $pressId, $offset, $limit, $total, &$sets));
+		HookRegistry::call('OAIDAO::getSets', [&$this, $pressId, $offset, $limit, $total, &$sets]);
 
 		$total = count($sets);
 		$sets = array_slice($sets, $offset, $limit);
@@ -128,7 +128,7 @@ class OAIDAO extends PKPOAIDAO {
 	function getSetPressSeriesId($pressSpec, $seriesSpec, $restrictPressId = null) {
 		$press = $this->_pressDao->getByPath($pressSpec);
 		if (!isset($press) || (isset($restrictPressId) && $press->getId() != $restrictPressId)) {
-			return array(0, 0);
+			return [0, 0];
 		}
 
 		$pressId = $press->getId();
@@ -143,7 +143,7 @@ class OAIDAO extends PKPOAIDAO {
 			}
 		}
 
-		return array($pressId, $seriesId);
+		return [$pressId, $seriesId];
 	}
 
 
@@ -159,7 +159,7 @@ class OAIDAO extends PKPOAIDAO {
 		$publicationFormatId = $row['data_object_id'];
 
 		$record->identifier = $this->oai->publicationFormatIdToIdentifier($publicationFormatId);
-		$record->sets = array(urlencode($press->getPath()) . ($series?':' . urlencode($series->getPath()):''));
+		$record->sets = [urlencode($press->getPath()) . ($series?':' . urlencode($series->getPath()):'')];
 
 		if ($isRecord) {
 			$publicationFormat = $this->_publicationFormatDao->getById($publicationFormatId);
@@ -183,13 +183,13 @@ class OAIDAO extends PKPOAIDAO {
 	 * @param $set string
 	 * @param $submissionId int Optional
 	 * @param $orderBy string UNFILTERED
-	 * @return ADORecordSet
+	 * @return Iterable
 	 */
 	function _getRecordsRecordSet($setIds, $from, $until, $set, $submissionId = null, $orderBy = 'press_id, data_object_id') {
 		$pressId = array_shift($setIds);
 		$seriesId = array_shift($setIds);
 
-		$params = array();
+		$params = [];
 		if ($pressId) $params[] = (int) $pressId;
 		if ($seriesId) $params[] = (int) $seriesId;
 		if ($submissionId) $params[] = (int) $submissionId;
@@ -199,7 +199,7 @@ class OAIDAO extends PKPOAIDAO {
 		if ($submissionId) $params[] = (int) $submissionId;
 
 		import('lib.pkp.classes.submission.PKPSubmission'); // STATUS_PUBLISHED
-		$result = $this->retrieve(
+		return $this->retrieve(
 			'SELECT	ms.last_modified AS last_modified,
 				pf.publication_format_id AS data_object_id,
 				p.press_id AS press_id,
@@ -241,8 +241,6 @@ class OAIDAO extends PKPOAIDAO {
 			ORDER BY ' . $orderBy,
 			$params
 		);
-
-		return $result;
 	}
 }
 
