@@ -3,8 +3,8 @@
 /**
  * @file plugins/importexport/native/NativeImportExportPlugin.inc.php
  *
- * Copyright (c) 2014-2020 Simon Fraser University
- * Copyright (c) 2003-2020 John Willinsky
+ * Copyright (c) 2014-2021 Simon Fraser University
+ * Copyright (c) 2003-2021 John Willinsky
  * Distributed under the GNU GPL v3. For full terms see the file docs/COPYING.
  *
  * @class NativeImportExportPlugin
@@ -84,31 +84,28 @@ class NativeImportExportPlugin extends ImportExportPlugin {
 		switch (array_shift($args)) {
 			case 'index':
 			case '':
-				$exportSubmissionsListPanel = new \PKP\components\listPanels\PKPSelectSubmissionsListPanel(
-					'exportSubmissionsListPanel',
-					__('plugins.importexport.native.exportSubmissionsSelect'),
+				$apiUrl = $request->getDispatcher()->url($request, ROUTE_API, $context->getPath(), 'submissions');
+				$submissionsListPanel = new \APP\components\listPanels\SubmissionsListPanel(
+					'submissions',
+					__('common.publications'),
 					[
-						'apiUrl' => $request->getDispatcher()->url(
-							$request,
-							ROUTE_API,
-							$context->getPath(),
-							'_submissions'
-						),
-						'canSelect' => true,
-						'canSelectAll' => true,
+						'apiUrl' => $apiUrl,
 						'count' => 100,
+						'getParams' => new stdClass(),
 						'lazyLoad' => true,
-						'selectorName' => 'selectedSubmissions[]',
 					]
 				);
-				$templateMgr->assign('exportSubmissionsListData', [
+				$submissionsConfig = $submissionsListPanel->getConfig();
+				$submissionsConfig['addUrl'] = '';
+				$submissionsConfig['filters'] = array_slice($submissionsConfig['filters'], 1);
+				$templateMgr->setState([
 					'components' => [
-						'exportSubmissionsListPanel' => $exportSubmissionsListPanel->getConfig()
-					]
+						'submissions' => $submissionsConfig,
+					],
+				]);
+				$templateMgr->assign([
+					'pageComponent' => 'ImportExportPage',
 				]);
-
-				$templateMgr->assign('currentcontext', $context);
-
 				$templateMgr->display($this->getTemplateResource('index.tpl'));
 				break;
 			case 'uploadImportXML':
@@ -131,11 +128,12 @@ class NativeImportExportPlugin extends ImportExportPlugin {
 				$json = new JSONMessage(true);
 				$json->setEvent('addTab', array(
 					'title' => __('plugins.importexport.native.results'),
-					'url' => $request->url(null, null, null, array('plugin', $this->getName(), 'import'), array('temporaryFileId' => $request->getUserVar('temporaryFileId'))),
+					'url' => $request->url(null, null, null, array('plugin', $this->getName(), 'import'), array('temporaryFileId' => $request->getUserVar('temporaryFileId'), 'csrfToken' => $request->getSession()->getCSRFToken())),
 				));
 				header('Content-Type: application/json');
 				return $json->getString();
 			case 'import':
+				if (!$request->checkCSRF()) throw new Exception('CSRF mismatch!');
 				AppLocale::requireComponents(LOCALE_COMPONENT_PKP_SUBMISSION);
 				$temporaryFileId = $request->getUserVar('temporaryFileId');
 				$temporaryFileDao = DAORegistry::getDAO('TemporaryFileDAO'); /* @var $temporaryFileDao TemporaryFileDAO */
@@ -169,7 +167,7 @@ class NativeImportExportPlugin extends ImportExportPlugin {
 				if (!empty($submissionsWarnings)) {
 					$templateMgr->assign('submissionsWarnings', $submissionsWarnings);
 				}
-				// If there are any submissions or validataion errors
+				// If there are any submissions or validation errors
 				// delete imported submissions.
 				if (!empty($submissionsErrors) || !empty($validationErrors)) {
 					// remove all imported submissions
@@ -294,17 +292,28 @@ class NativeImportExportPlugin extends ImportExportPlugin {
 					return;
 				}
 
-				$filter = 'native-xml=>monograph';
-				// is this monographs import:
+				$request = Application::get()->getRequest();
+				// Set global user
+				if (!$request->getUser()) {
+					Registry::set('user', $user);
+				}
+				// Set global context
+				if (!$request->getContext()) {
+					HookRegistry::register('Router::getRequestedContextPaths', function (string $hook, array $args) use ($press): bool {
+						$args[0] = [$press->getPath()];
+						return false;
+					});
+					$router = new PageRouter();
+					$router->setApplication(Application::get());
+					$request->setRouter($router);
+				}
+
 				$xmlString = file_get_contents($xmlFile);
 				$document = new DOMDocument();
 				$document->loadXml($xmlString);
-				if (in_array($document->documentElement->tagName, array('monograph', 'monographs'))) {
-					$filter = 'native-xml=>monograph';
-				}
 				$deployment = new NativeImportExportDeployment($press, $user);
 				$deployment->setImportPath(dirname($xmlFile));
-				$content = $this->importSubmissions($xmlString, $filter, $deployment);
+				$content = $this->importSubmissions($xmlString, $deployment);
 				$validationErrors = array_filter(libxml_get_errors(), function($a) {
 					return $a->level == LIBXML_ERR_ERROR || $a->level == LIBXML_ERR_FATAL;
 				});
@@ -348,13 +357,18 @@ class NativeImportExportPlugin extends ImportExportPlugin {
 						$foundErrors = true;
 					}
 				}
-				// If there are any data or validataion errors
+				// If there are any data or validation errors
 				// delete imported objects.
 				if ($foundErrors || !empty($validationErrors)) {
-					// remove all imported issues and sumissions
+					// remove all imported issues and submissions
 					foreach (array_keys($errorTypes) as $assocType) {
 						$deployment->removeImportedObjects($assocType);
 					}
+					echo __('plugins.importexport.common.validationErrors') . "\n";
+					$i = 0;
+					foreach ($validationErrors as $validationError) {
+						echo ++$i . '. Line: ' . $validationError->line . ' Column: ' . $validationError->column . ' > ' . $validationError->message . "\n";
+					}
 				}
 				return;
 			case 'export':
@@ -403,7 +417,7 @@ class NativeImportExportPlugin extends ImportExportPlugin {
 				$sticky = null;
 				continue;
 			}
-			if (!starts_with($arg, '--')) {
+			if (substr($arg, 0, 2) != '--') {
 				$newArgs[] = $arg;
 				continue;
 			}
