@@ -3,8 +3,8 @@
 /**
  * @file plugins/themes/default/DefaultThemePlugin.inc.php
  *
- * Copyright (c) 2014-2020 Simon Fraser University
- * Copyright (c) 2003-2020 John Willinsky
+ * Copyright (c) 2014-2021 Simon Fraser University
+ * Copyright (c) 2003-2021 John Willinsky
  * Distributed under the GNU GPL v3. For full terms see the file docs/COPYING.
  *
  * @class DefaultThemePlugin
@@ -93,64 +93,27 @@ class DefaultThemePlugin extends ThemePlugin {
 		// Store additional LESS variables to process based on options
 		$additionalLessVariables = array();
 
-		// Load fonts from Google Font CDN
-		// To load extended latin or other character sets, see:
-		// https://www.google.com/fonts#UsePlace:use/Collection:Noto+Sans
-		if (Config::getVar('general', 'enable_cdn')) {
-
-			if ($this->getOption('typography') === 'notoSerif') {
-				$this->addStyle(
-					'fontNotoSerif',
-					'//fonts.googleapis.com/css?family=Noto+Serif:400,400i,700,700i',
-					array('baseUrl' => '')
-				);
-				$additionalLessVariables[] = '@font: "Noto Serif", -apple-system, BlinkMacSystemFont, "Segoe UI", "Roboto", "Oxygen-Sans", "Ubuntu", "Cantarell", "Helvetica Neue", sans-serif;';
-
-			} elseif (strpos($this->getOption('typography'), 'notoSerif') !== false) {
-				$this->addStyle(
-					'fontNotoSansNotoSerif',
-					'//fonts.googleapis.com/css?family=Noto+Sans:400,400i,700,700i|Noto+Serif:400,400i,700,700i',
-					array('baseUrl' => '')
-				);
-
-				// Update LESS font variables
-				if ($this->getOption('typography') == 'notoSerif_notoSans') {
-					$additionalLessVariables[] = '@font-heading: "Noto Serif", serif;';
-				} elseif ($this->getOption('typography') == 'notoSans_notoSerif') {
-					$additionalLessVariables[] = '@font: "Noto Serif", serif;@font-heading: "Noto Sans", serif;';
-				}
-
-			} elseif ($this->getOption('typography') == 'lato') {
-				$this->addStyle(
-					'fontLato',
-					'//fonts.googleapis.com/css?family=Lato:400,400i,900,900i',
-					array('baseUrl' => '')
-				);
-				$additionalLessVariables[] = '@font: Lato, sans-serif;';
-
-			} elseif ($this->getOption('typography') == 'lora') {
-				$this->addStyle(
-					'fontLora',
-					'//fonts.googleapis.com/css?family=Lora:400,400i,700,700i',
-					array('baseUrl' => '')
-				);
-				$additionalLessVariables[] = '@font: Lora, serif;';
-
-			} elseif ($this->getOption('typography') == 'lora_openSans') {
-				$this->addStyle(
-					'fontLoraOpenSans',
-					'//fonts.googleapis.com/css?family=Lora:400,400i,700,700i|Open+Sans:400,400i,700,700i',
-					array('baseUrl' => '')
-				);
-				$additionalLessVariables[] = '@font: "Open Sans", sans-serif;@font-heading: Lora, serif;';
-
-			} else {
-				$this->addStyle(
-					'fontNotoSans',
-					'//fonts.googleapis.com/css?family=Noto+Sans:400,400italic,700,700italic',
-					array('baseUrl' => '')
-				);
+		if ($this->getOption('typography') === 'notoSerif') {
+			$this->addStyle('font', 'styles/fonts/notoSerif.less');
+			$additionalLessVariables[] = '@font: "Noto Serif", -apple-system, BlinkMacSystemFont, "Segoe UI", "Roboto", "Oxygen-Sans", "Ubuntu", "Cantarell", "Helvetica Neue", sans-serif;';
+		} elseif (strpos($this->getOption('typography'), 'notoSerif') !== false) {
+			$this->addStyle('font', 'styles/fonts/notoSans_notoSerif.less');
+			if ($this->getOption('typography') == 'notoSerif_notoSans') {
+				$additionalLessVariables[] = '@font-heading: "Noto Serif", serif;';
+			} elseif ($this->getOption('typography') == 'notoSans_notoSerif') {
+				$additionalLessVariables[] = '@font: "Noto Serif", serif;@font-heading: "Noto Sans", serif;';
 			}
+		} elseif ($this->getOption('typography') == 'lato') {
+			$this->addStyle('font', 'styles/fonts/lato.less');
+			$additionalLessVariables[] = '@font: Lato, sans-serif;';
+		} elseif ($this->getOption('typography') == 'lora') {
+			$this->addStyle('font', 'styles/fonts/lora.less');
+			$additionalLessVariables[] = '@font: Lora, serif;';
+		} elseif ($this->getOption('typography') == 'lora_openSans') {
+			$this->addStyle('font', 'styles/fonts/lora_openSans.less');
+			$additionalLessVariables[] = '@font: "Open Sans", sans-serif;@font-heading: Lora, serif;';
+		} else {
+			$this->addStyle('font', 'styles/fonts/notoSans.less');
 		}
 
 		// Update colour based on theme option
@@ -164,20 +127,15 @@ class DefaultThemePlugin extends ThemePlugin {
 
 		// Pass additional LESS variables based on options
 		if (!empty($additionalLessVariables)) {
-			$this->modifyStyle('stylesheet', array('addLessVariables' => join($additionalLessVariables)));
+			$this->modifyStyle('stylesheet', array('addLessVariables' => join("\n", $additionalLessVariables)));
 		}
 
 		$request = Application::get()->getRequest();
 
 		// Load icon font FontAwesome - http://fontawesome.io/
-		if (Config::getVar('general', 'enable_cdn')) {
-			$url = 'https://cdnjs.cloudflare.com/ajax/libs/font-awesome/4.7.0/css/font-awesome.css';
-		} else {
-			$url = $request->getBaseUrl() . '/lib/pkp/styles/fontawesome/fontawesome.css';
-		}
 		$this->addStyle(
 			'fontAwesome',
-			$url,
+			$request->getBaseUrl() . '/lib/pkp/styles/fontawesome/fontawesome.css',
 			array('baseUrl' => '')
 		);
 
@@ -194,27 +152,19 @@ class DefaultThemePlugin extends ThemePlugin {
 
 			$this->addStyle(
 				'homepageImage',
-				'.pkp_structure_head { background: inherit center / cover no-repeat url("' . $homepageImageUrl . '");}',
+				'.pkp_structure_head { background: center / cover no-repeat url("' . $homepageImageUrl . '");}',
 				['inline' => true]
 			);
 		}
 
 		// Load jQuery from a CDN or, if CDNs are disabled, from a local copy.
 		$min = Config::getVar('general', 'enable_minified') ? '.min' : '';
-		$request = Application::get()->getRequest();
-		if (Config::getVar('general', 'enable_cdn')) {
-			$jquery = '//ajax.googleapis.com/ajax/libs/jquery/' . CDN_JQUERY_VERSION . '/jquery' . $min . '.js';
-			$jqueryUI = '//ajax.googleapis.com/ajax/libs/jqueryui/' . CDN_JQUERY_UI_VERSION . '/jquery-ui' . $min . '.js';
-		} else {
-			// Use OJS's built-in jQuery files
-			$jquery = $request->getBaseUrl() . '/lib/pkp/lib/vendor/components/jquery/jquery' . $min . '.js';
-			$jqueryUI = $request->getBaseUrl() . '/lib/pkp/lib/vendor/components/jqueryui/jquery-ui' . $min . '.js';
-		}
+		$jquery = $request->getBaseUrl() . '/lib/pkp/lib/vendor/components/jquery/jquery' . $min . '.js';
+		$jqueryUI = $request->getBaseUrl() . '/lib/pkp/lib/vendor/components/jqueryui/jquery-ui' . $min . '.js';
 		// Use an empty `baseUrl` argument to prevent the theme from looking for
 		// the files within the theme directory
 		$this->addScript('jQuery', $jquery, array('baseUrl' => ''));
 		$this->addScript('jQueryUI', $jqueryUI, array('baseUrl' => ''));
-		$this->addScript('jQueryTagIt', $request->getBaseUrl() . '/lib/pkp/js/lib/jquery/plugins/jquery.tag-it.js', array('baseUrl' => ''));
 
 		// Load Bootsrap's dropdown
 		$this->addScript('popper', 'js/lib/popper/popper.js');
